unit wcsitef;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
  StdCtrls, ComCtrls, ExtCtrls, Controls,Dialogs,
  webcompr, CompLHA ;
  
type
  TwcSiteForm = class(TForm)
    Panel2: TPanel;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    UploadPath: TEdit;
    OriginPath: TEdit;
    Password: TEdit;
    Panel1: TPanel;
    GatherButton: TButton;
    deployButton: TButton;
    RefreshButton: TButton;
    Status: TButton;
    SitesList: TComboBox;
    StatusBar1: TStatusBar;
    msg: TMemo;
    CompLHA1: TCompLHA;
    WebCompress: TWebCompress;
    Panel4: TPanel;
    GatherFileList: TListBox;
    Label5: TLabel;
    Panel5: TPanel;
    Label6: TLabel;
    UserApp: TEdit;
    Responsive: TCheckBox;
    cbNoRenameEXEs: TCheckBox;
    procedure StatusClick(Sender: TObject);
    procedure deployButtonClick(Sender: TObject);
    procedure GatherButtonClick(Sender: TObject);
    procedure UploadPathExit(Sender: TObject);
    procedure OriginPathExit(Sender: TObject);
    procedure PasswordExit(Sender: TObject);
    procedure SitesListChange(Sender: TObject);
    procedure WebCompressAfterChangeProperty(const propertyname: String);
    procedure SetList(UIList,WCList: TStrings);
    procedure WebCompressAfterDeploy(processed: Integer; files: TStrings);
    procedure WebCompressAfterGather(files: TStrings);
    procedure FormCreate(Sender: TObject);
    procedure RefreshButtonClick(Sender: TObject);
    procedure SitesListExit(Sender: TObject);
    procedure UserAppExit(Sender: TObject);
    procedure WebCompressUserSignal(Sender: TObject);
    procedure ResponsiveClick(Sender: TObject);
    procedure cbNoRenameEXEsClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  wcSiteForm: TwcSiteForm;

implementation

{$R *.DFM}

uses inifiles;

function RightOfEqual(const expression: string): string;
var epos : Integer;
begin
  result := expression;
  epos := pos('=',result);
  if epos > 0 then
    result := copy(result,epos+1,MaxLongInt);
end;


function FilenameOnly(const filename: string): string;
var p: smallint;
begin
  result := filename;
  p := pos('.',result);
  if p = 0 then exit;
  result := copy(result,1,p-1);
end;

procedure TwcSiteForm.StatusClick(Sender: TObject);
begin
   msg.lines.clear;
   msg.lines.addstrings(WebCompress.Status);
end;

procedure TwcSiteForm.deployButtonClick(Sender: TObject);
begin
   WebCompress.Deploy;
end;

procedure TwcSiteForm.GatherButtonClick(Sender: TObject);
begin
   WebCompress.Gather;
end;

procedure TwcSiteForm.UploadPathExit(Sender: TObject);
begin
  WebCompress.UploadPath :=  (sender as TEdit).Text;
end;

procedure TwcSiteForm.OriginPathExit(Sender: TObject);
begin
  WebCompress.OriginPath :=  (sender as TEdit).Text;
end;

procedure TwcSiteForm.PasswordExit(Sender: TObject);
begin
  WebCompress.Password := (sender as TEdit).Text;
end;

procedure TwcSiteForm.SitesListChange(Sender: TObject);
begin
  GatherFileList.Items.clear;
  WebCompress.Site := SitesList.text;
end;

procedure TwcSiteForm.SetList(UIList,WCList: TStrings);
var listcount: Integer;
begin
  with UIList do
  begin
    Clear;
    for listcount := 0 to pred(WCList.count) do
       Add(RightofEqual(WCList[listcount]));
  end;
end;



procedure TwcSiteForm.WebCompressAfterChangeProperty(
  const propertyname: String);
begin
  if propertyname='Sites' then
  begin
    SetList(SitesList.Items,WebCompress.Sites);
    exit;
  end else if propertyname = 'GatherList' then
  begin
    SetList(GatherFileList.Items,WebCompress.GatherList);
    exit;
  end;

  { for the rest, we could do one by one, but let's be lazy... }
  SitesList.Text := WebCompress.Site;
  UploadPath.Text :=WebCompress.UploadPath;
  Password.Text := WebCompress.Password;
  OriginPath.Text := WebCompress.OriginPath;
  cbNoRenameEXEs.checked := not WebCompress.RenameEXEs;
end;

procedure TwcSiteForm.WebCompressAfterDeploy(processed: Integer;
  files: TStrings);
begin
  StatusClick(nil); { only at this late stage do we have a status to show }
  StatusBar1.SimpleText := 'Deployed '+IntToStr(processed)+' archives at '+DateTimeToStr(Now);
end;

procedure TwcSiteForm.WebCompressAfterGather(files: TStrings);
begin
  StatusClick(nil); { only at this late stage do we have a status to show }
  StatusBar1.SimpleText := 'Gathered '+IntToStr(files.count)+' files at '+DateTimeToStr(Now);
end;

procedure TwcSiteForm.FormCreate(Sender: TObject);
begin
  WebCompress.IniFilename := ExtractFilePath(paramstr(0))+FileNameOnly(extractFilename(paramstr(0)))+'.ini';
  RefreshButtonClick(nil); // get all updated
  Application.OnIdle := WebCompress.ApplicationIdle;
{$IFNDEF SPISVERSION} 
  msg.lines.add('1. If this app starts processing a Deploy/Gather operation, then seems to hang, you probably have an unregistered version of TCompLHA -- look for a reminder dialog and click its OK button...');
  msg.lines.add('2. If this app seems slow to begin processing during FTP remote control from WebQueue, move the mouse over it. On Win95 in particular, the "Idle" event it uses to detect signals can be very infrequent (say, once a minute)');
{$ENDIF}
end;

procedure TwcSiteForm.RefreshButtonClick(Sender: TObject);
var ini : TInifile;
begin
  if WebCompress.Update then
  begin
    SetList(SitesList.Items,WebCompress.Sites);
    SitesList.Text := WebCompress.Site;
    if WebCompress.IniFilename<>'' then
    begin
      ini := TIniFile.create(WebCompress.IniFilename);
      with ini do
      try
        UserApp.Text := readString('WebCompress','UserApp','');
        UserAppExit(nil);
        Responsive.checked := ReadBool('WebCompress','Responsive',False);
        WebCompress.Responsive := Responsive.checked;
      finally
        ini.free;
      end;
    end;
  end else
    Showmessage('Unable to initialise WebCompress -- are CompLHA and INIfileName set?');
end;

procedure TwcSiteForm.SitesListExit(Sender: TObject);
var sl: TStringList;
    count: Integer;
begin
 with (Sender as TComboBox) do
     if ItemIndex < 0 then { new site name typed in }
     begin
        if MessageDlg('Create new site called: '+Text, mtConfirmation,
            [mbYes, mbNo], 0)=id_Yes then
        begin
         Items.add(Text);
         sl := TStringList.create;
         try
           for count := 0 to pred(Items.count) do
               sl.add(IntToStr(count)+'='+Items[count]);
           WebCompress.Sites.assign(sl);
         finally
           sl.free;
         end;
         WebCompress.Site := Text;
        end else
        begin
          ItemIndex := 0;
        end;
     end;
end;

procedure TwcSiteForm.UserAppExit(Sender: TObject);
var ini: TIniFile;
begin
   if UserApp.text='' then
     WebCompress.UserSignalFilename:=''
   else
     WebCompress.UserSignalFilename:='UserApp.sig';
   if WebCompress.IniFilename<>'' then
   begin
     ini := TIniFile.create(WebCompress.IniFilename);
     with ini do
     try
       writeString('WebCompress','UserApp',UserApp.Text);
     finally
       ini.free;
     end;
   end;
end;

procedure TwcSiteForm.WebCompressUserSignal(Sender: TObject);
begin
   if UserApp.Text='' then exit;
   WinExec (Pchar(UserApp.Text), SW_SHOW);
end;

procedure TwcSiteForm.ResponsiveClick(Sender: TObject);
var ini: TIniFile;
begin
  WebCompress.Responsive := Responsive.Checked;
  if WebCompress.IniFilename<>'' then
  begin
    ini := TIniFile.create(WebCompress.IniFilename);
    with ini do
    try
      WriteBool('WebCompress','Responsive',Responsive.checked);
    finally
      ini.free;
    end;
  end;

end;

procedure TwcSiteForm.cbNoRenameEXEsClick(Sender: TObject);
begin
  WebCompress.RenameEXEs := not cbNoRenameEXEs.checked;
end;

end.
