unit webcompr;

(*  TWebCompress V2.1
   (c) 1997-2008 South Pacific Information Services Ltd
    ALL RIGHTS RESERVED     software@spis.co.nz

    V2.1 July 2008 - added section-specific INI and PropertyChange support
    for RenameEXEs

Legal stuff:
This source is provided in support of our remote administration
applications for Web sites, such as SPIS WebQueue.
It may not be used or modified for the development of similar components
or applications for Web site developers or users without the express
permission of SPIS Ltd.

TWebCompress is a compression automation and administration
component for Web sites. It requires our TCompLHA component
(http://www.spis.co.nz/complha.htm) to provide the compression
services. It uses FTP-based remote control and an INI file to
manage either:

* Uploaded archives to be deployed on the site (or sites) which
  your application is managing (e.g. new pages, chunk files,
  INI files, images, DB files etc)
* Files from the site which are to be "gathered" into an
  archive for subsequent downloading (e.g. stats, order
  information, messages, customer information, site backup)

Full verification of archives is implemented, to prevent failures
due to erroneous uploads. File encryption is also supported.

You'll probably also want to obtain our WebQueue application, which is
designed to interact with TWebCompress via FTP -- it will
automatically create an FTP update files for TWebCompress to
deploy, or specify files for TWebCompress to gather, then fetch
the resulting archive. WebQueue is NOT essential, because
TWebCompress will process any archives made by the popular
LHA freeware utility. HOWEVER, once you've tried WebQueue, we
doubt you've ever go back to LHA!

Thus, the administration of all remote site maintenance can be
reliably automated with a combination of TWebCompress+TCompLHA
and optionally, WebQueue.

This is V2.0, which is adequate for managing a site or set of sites
based on one Web application. It can be used to manage sites based
on multiple Web automation applications provided you don't need to
overwrite files which the other EXEs might have open. The reason for
this is that, while you can use events on this component to tell THIS
application -- the one it is part of -- that files may have just changed,
there is no inherent interface to notify other ones. However, you could
add this functionality yourself in response to the event if you chose.

See WebDeply.dpr for a working example configured as a ready-made
application. WebCompr.txt contains full component documentation,
and WebQueue.hlp has information on the WebQueue utility
(http://www.spis.co.nz/webcentr/webqueue.htm).

For those working with HREF Tool's Corp's excellent WebHub site automation
framework (http://www.href.com), you'll want to work with the TWHCompress
component in the whcompr.pas/.txt and wcpanel.* files, as together they
provide a drop-in WebHub UI panel which includes HTML remote control features.

*)

interface

uses
  Windows, SysUtils, Classes, CompLHA;

type
  TOnDeployEvent = procedure(files: TStrings) of Object;
  TAfterDeployEvent = procedure(processed: Integer;files: TStrings) of Object;
  TOnExpandEvent = procedure(var filename: string; status: TStrings) of Object;
  TAfterExpandEvent = procedure(const filename: string; filesprocessed, status: TStrings) of Object;
  TAfterChangePropertyEvent = procedure(const propertyname: string) of Object;

  TWebCompress = class(TComponent)
  private
   fIniFileName: string;
   fSites: TStringList;
   fSite: string;
   fUserSignalFileName: string;
   fUploadPath: string;
   fAppID: string;
   fPassword: string;
   fOriginPath: string;
   fCheckMessages: TStringList;
   fStatus: TStrings;
   fGatherList: TStringList;
   fCompLHA: TCompLHA;
   fResponsive: Boolean;
   fRenameEXEs: Boolean;
   fAfterChangeProperty: TAfterChangePropertyEvent;
   fOnDeploy: TOnDeployEvent;
   fAfterDeploy: TAfterDeployEvent;
   fOnGather: TOnDeployEvent;
   fAfterGather: TOnDeployEvent;
   fOnExpand: TOnExpandEvent;
   fAfterExpand: TAfterExpandEvent;
   fSaveCheckFile: TCompLHAFilenameCheckEvent;
   fScanningSites: Boolean;
   fOnUserSignal : TNotifyEvent;
   function GetStatus: TStrings;
   procedure CheckFile(const originalFilePath: string;var newFilepath: String; dateTime: Longint;
           mode: TCompLHAProcessMode);
   function ExpandOneArchive(const filename: string;results: TStrings): Boolean;
   procedure checkForGatherParams(const spath: string); { are they trying to tell us something via the file contents? }
   function StatusPath: string;

  protected

    procedure ReadFromIniFile;
    procedure WriteToIniFile;
    procedure ReadSiteFromINIFile;
    function init: Boolean;
    procedure SetStatusStrings(Value: TStringList);
    procedure SetGatherStrings(Value: TStringList);
    procedure SetSites(Value: TStringList);
    procedure SetSite(Value: string);
    procedure SetUserSignal(const value: string);
    procedure SetOriginPath(const Value: String);
    procedure SetAppID(const Value: String);
    procedure SetPassWord(const Value: String);
    procedure SetUpLoadPath(const Value: string);
    procedure SetRenameEXEs(const Value: Boolean);
    procedure DoPropertyChange(const propertyname: string);

  public
   constructor Create(AOwner: TComponent); override;
   destructor  Destroy; override;
   procedure Deploy;
   procedure Gather;
   procedure ApplicationIdle(Sender: TObject; var Done: Boolean);
   function Update: Boolean;

  published
   property UploadPath: string read fUploadPath write SetUploadPath stored False;
   property OriginPath: string read fOriginPath write SetOriginPath stored False;
   property AppID: string read fAppID write SetAppID stored False;
   property Password: string read fPassword write SetPassword stored False;
   property Site: string read fSite write SetSite stored false;
   property Sites: TStringList read fSites write SetSites stored false;
   property CompLHA: TCompLHA read fCompLHA write fCompLHA;
   property GatherList: TStringList read fgatherList write SetGatherStrings stored false;
   property Responsive: Boolean read fResponsive write fResponsive;
   property RenameEXEs: Boolean read fRenameEXEs write SetRenameEXEs default True;
   property OnDeploy:  TOnDeployEvent read fOnDeploy write fOnDeploy;
   property AfterDeploy: TAfterDeployEvent read fAfterDeploy write fAfterDeploy;
   property AfterChangeProperty:  TAfterChangePropertyEvent read fAfterChangeProperty write fAfterChangeProperty;
   property OnExpand: TOnExpandEvent read fOnExpand write fOnExpand;
   property AfterExpand: TAfterExpandEvent read fAfterExpand write fAfterExpand;
   property OnGather:  TOnDeployEvent read fOnGather write fOnGather;
   property AfterGather:  TOnDeployEvent read fAfterGather write fAfterGather;
   property Status: TStrings read GetStatus;
   property UserSignalFileName: string read fUserSignalFileName write SetUserSignal;
   property IniFileName: string read fIniFileName write fIniFileName;
   property OnUserSignal: TNotifyEvent read fOnUserSignal write fOnUserSignal;
  end;

procedure Register;

implementation

uses inifiles;

const { INI settings }
      SITENAME='Site'; { under [WebCompress1] }
      SITESNAME='Sites'; { e.g. [WebCompress1.Sites] }
      USERSIGNALNAME='UserSignal';

      PASSWORDNAME='Password'; { under [WebCompress1.Sitename] }
      APPIDNAME='AppID';
      ORIGINPATHNAME='OriginPath';
      UPLOADPATHNAME='UploadPath';
      RENAMEEXESNAME = 'RenameEXEs';

      GATHERLISTNAME='GatherList'; { e.g. WebCompress1.Sitename.GatherList }

      SELECTSITECOMMAND = 'SelectSite';
      EXECUTECOMMAND='Execute';
      GATHERCOMMAND='Gather';
      DEPLOYCOMMAND='Deploy';
      REFRESHCOMMAND='Refresh';


// handles "..\" elimination to ensure minimal pathnames
// e.g. ConcatPath('c:\path\here','..\..\file.txt') is 'c:\file.txt'
// Will not concat if Filename is an absolute path
function ConcatPath(const Pathname,Filename:string):string;
var
  left:string;
  count,spos:integer;
begin
  result:=filename;
  if (length(filename)>1) and
    ((filename[2]=':') or (filename[1]='\')) then
    exit;  //filename already specifies a drive or absolute path (unc or just \)
  left := Pathname;
  if copy(left,Length(left),1)='\' then
     left := copy(left,1,Length(left)-1);
  while (copy(result,1,3)='..\') do
  begin
    spos:=pos('\',left);
    if spos>0 then
    begin
      for count:=length(left) downto spos do
        if left[count]='\' then
        begin
          left := copy(left,1,pred(count));
          break;
        end;
    end else
      break;
  end;
  if copy(left,Length(left),1)<>'\' then
    left := left + '\';
  result:=left+result;
end;

function RightOfEqual(const expression: string): string;
var epos : Integer;
begin
  result := expression;
  epos := pos('=',result);
  if epos > 0 then
    result := copy(result,epos+1,MaxLongInt);
end;

{ We'll *always* prepend a non-blank path provided Filename doesn't have a
  leading UNC or drive designator. This differs from ConCatPath alone which
  skips the prepend if the filename already has an absolute path.
}
function ForceConCatPath(const Path,FileName:String):String;
begin
 result := Filename;
 if Length(result)>0 then
 begin
   if Path<>'' then { may need to adjust filename }
   begin
     if result[1]='\' then    { is a leading \ }
       if copy(result,1,2)<>'\\' then { but not a UNC path }
         result :=copy(result,2,999); { strip the leading \ }
   end;
 end;
 result := ConcatPath(Path,Result); { tidily prepend... }
end;

function isDir(const path: string): Boolean;
var attr: Integer;
begin
  attr:=FileGetAttr(path);
  result := (attr>=0) and ((attr AND faDirectory) > 0);
end;

function TWebCompress.Update: Boolean;
begin
  result := init;
  if result then
    ReadFromIniFile;
end;

procedure TWebCompress.CheckFile(const originalFilePath: string;var newFilepath: String; dateTime: Longint;
           mode: TCompLHAProcessMode);
var ext, diskpath: string;
    relative: Boolean;
begin
  { start by ensuring all access to and from is relative to (or at least
    matching) OriginPath
  }
  if OriginPath<>'' then { ok, do some work -- expand OR compress }
  begin
     relative := NewFilePath[1]<>'\';
     if mode = cmLHAExpand then
     begin
       if (pos(':',OriginPath)>0) and (not relative) then { get drive }
         NewFilepath := ForceConcatPath(copy(OriginPath,1,pos(':',OriginPath)),NewFilePath);
       diskPath := newFilePath;
     end else
       diskPath := OriginalFilePath; { where we're coming FROM }
     ext := copy(diskpath,1,Length(OriginPath));
     if CompareText(ext,OriginPath)<>0 then { not in the same path, too bad... }
     begin { illegal }
         if not CompLHA.safemode then { ignore if Verify step... }
            fCheckMessages.add('=== '+diskPath+' NOT expanded -- illegal path!');
         newFilePath := CompLHASkipFlag; { no way, Jose }
         exit;
     end;
  end;

  if (mode=cmLHAExpand) and RenameEXEs then
  begin
    ext := UpperCase(ExtractFileExt(originalFilePath));
    if (ext='.EXE') or (ext='.OCX') or (ext='.DLL') then
    begin
       ext := copy(ext,1,3)+'_'; { EXE --> EX_ }
       newfilepath := ChangeFileExt(newfilepath,ext);
       if not CompLHA.safemode then { ignore if Verify step... }
          fCheckMessages.add('=== '+originalFilePath+' RENAMED to '+newfilePath);
    end;
  end;

  if assigned(FSaveCheckFile) then
     FSaveCheckFile(originalFilePath,newFilePath,dateTime,mode);
end;

constructor TWebCompress.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  fStatus := TStringList.create;
  fGatherList := TStringList.Create;
  fSites := TStringList.Create;
  fCheckMessages := TStringList.Create;
  RenameExes := True;
end;

destructor TWebCompress.Destroy;
begin
  fStatus.free;
  fGatherList.free;
  fSites.free;
  fCheckMessages.free;
  inherited Destroy;
end;

function TWebCompress.init: Boolean;
begin
  result := assigned(CompLHA) and (iniFilename<>'');
end;

procedure TWebCompress.DoPropertyChange(const propertyname: string);
begin
  if fScanningSites then exit; { ignore when we're looking through sites in the idle loop }
  if assigned(fAfterChangeProperty) then
    fAfterChangeProperty(propertyname);
end;

procedure TWebCompress.SetSites(Value: TStringList);
begin
  fSites.assign(value);
  WriteToINIFile;
  DoPropertyChange('Sites');
end;

procedure TWebCompress.SetSite(Value: string);
var count: Integer;
    newsite: string;
begin
  newsite := '';
  for count := 0 to pred(Sites.count) do
  begin
    if CompareText(RightOfEqual(Sites[count]),value)=0 then
    begin
       newsite := RightofEqual(Sites[count]);
       break;
    end;
  end;
  if fSite<>newsite then
  begin
   fSite := newsite;
   DoPropertyChange('Site');
   ReadSiteFromINIFile; { get all properties up to date }
  end;
end;

procedure TWebCompress.SetAppID(const Value: String);
begin
  fAppID := value;
  WriteToIniFile;
  DoPropertyChange('AppID');
end;

procedure TWebCompress.SetPassword(const Value: String);
begin
  fPassword := value;
  WriteToIniFile;
  DoPropertyChange('Password');
end;


procedure TWebCompress.SetUploadPath(const Value: String);
begin
  fUploadPath := value;
  WriteToIniFile;
  DoPropertyChange('UploadPath');
end;

procedure TWebCompress.SetUserSignal(const Value: String);
begin
  fUserSignalFileName := value;
  WriteToIniFile;
  DoPropertyChange('UserSignal');
end;

procedure TWebCompress.SetOriginPath(const Value: String);
begin
  fOriginPath := value;
  WriteToIniFile;
  DoPropertyChange('OriginPath');
end;

procedure TWebCompress.SetRenameEXEs(const Value: Boolean);
begin
  fRenameEXEs := value;
  WriteToIniFile;
  DoPropertyChange('RenameEXEs');
end;


function TWebCompress.StatusPath: string;
begin
  result := ForceConcatPath(UploadPath,'STATUS.TXT');
end;

procedure TWebCompress.SetStatusStrings(Value: TStringList);
begin
 fStatus.Assign(Value);
 if FileExists(StatusPath) then
   DeleteFile(StatusPath);
end;

procedure TWebCompress.SetGatherStrings(Value: TStringList);
begin
  fGatherList.Assign(Value);
  WriteToINIFile;
  DoPropertyChange('GatherList');
end;

procedure WriteSection(ini: TInifile;const sectionName: string; list: TStrings);
var count: Integer;
begin
  with ini do
  begin       
    EraseSection(sectionName);
    for count := 0 to pred(list.count) do
      WriteString(sectionName,list.names[count],RightofEqual(list[count]));
  end;
end;

procedure TWebCompress.WriteToIniFile;
var ini: TIniFile;
    section: string;
begin
  if (not init) or (site='') then exit;
  ini := TIniFile.create(iniFileName);
  with ini do
  try
    writeString(self.name,SiteName,Site);
    writeString(self.name,UserSignalName,UserSignalFileName);
    Section := Self.name+'.'+Site;
    writeString(section,OriginPathName,OriginPath);
    writeString(section,UploadPathName,UploadPath);
    writeString(section,AppIDName,AppID);
    writeString(section,PasswordName,Password);
    writeBool(section,RenameEXEsName,RenameEXEs);
    writeSection(ini,self.name+'.'+site+'.'+GatherListName,fGatherList);
    writeSection(ini,self.name+'.'+SitesName,fSites);
  finally
    free;
  end;
end;

procedure TWebCompress.ReadFromIniFile;
var ini: TIniFile;
begin
  if not init then exit;
  ini := TIniFile.create(iniFileName);
  with ini do
  try
    ReadSectionvalues(self.name+'.'+SitesName,fsites);
    ReadString(self.name,UserSignalName,UserSignalFileName);
    ReadString(self.name,sitename,site); { default/current site, never empty if any defined... }
    if (site='') and (fsites.count >0) then // catchall
      site := rightofEqual(fsites[0]);
    // reading/setting it forces ReadSiteFromINIfile below...
  finally
    free;
  end;
end;

procedure TWebCompress.ReadSiteFromINIFile;
var ini: TIniFile;
    Section: string;
begin
  if (not init) or (Site='') then
     exit;
  ini := TIniFile.create(iniFileName);
  with ini do
  try
    Section := Self.name+'.'+Site;
    fOriginPath := readString(section,OriginPathName,'');
    fUploadPath := readString(section,UploadPathName,'');
    fAppID := ReadString(section,AppIDName,'');
    fPassword := ReadString(section,PasswordName,'');
    fRenameEXEs := ReadBool(section,RenameExesName,True);
    fGatherList.clear;
    readSectionValues(self.name+'.'+site+'.'+GatherListName,fGatherList);
    DoPropertyChange('GatherList');
    DoPropertyChange('OriginPath');
    DoPropertyChange('UploadPath');
    DoPropertyChange('AppID');
    DoPropertyChange('Password');
    DoPropertyChange('RenameEXEs');
  finally
    free;
  end;
end;


procedure TWebCompress.checkForGatherParams(const spath: string);
var lines: TStringList;
    count: Integer;
    line: string;
    cleared: Boolean;
begin { check if signal file has CONTENT, being GatherList settings... }
  lines := TStringList.Create;
  cleared := False;
  try
    try
      lines.LoadFromFile(spath);
    except on Exception do exit;
    end;
    for count := 0 to pred(lines.count) do
    begin
       line := lines[count];
       if line='' then continue;
       if not Cleared then
          fGatherList.clear;
       Cleared := True;
       fGatherList.add(inttostr(succ(count))+'='+line); { put the (presumed) paths into the list }
    end;
    if Cleared then { changed GatherList }
    begin
      WriteToINIFile;
      DoPropertyChange('GatherList');
    end;
  finally
    lines.free;
  end;
end;

procedure TWebCompress.ApplicationIdle(Sender: TObject;
  var Done: Boolean);
var spath, savesite: string;
    sitecount, fcount: Integer;
begin

  if Responsive then
  begin
   Done := False;
   sleep(100);
  end else { easy on the CPU, but veerry slow to respond }
   Done := True;

  try
   fScanningSites := True; { suspend AfterPropertyChange calls for now... }
   savesite := fSite;
   for sitecount := 0 to pred(Sites.count) do
   begin
     Site := RightofEqual(Sites[sitecount]); { load settings from INI }
     if not isDir(UploadPath) then continue; { ignore bad paths }
     for fcount := 0 to 3 do
     begin
       case fcount of
           0: spath := ForceConcatPath(Uploadpath,DeployCommand+'.SIG');
           1: spath := ForceConcatPath(Uploadpath,GatherCommand+'.SIG');
           2: spath := ForceConcatPath(Uploadpath,RefreshCommand+'.SIG');
           3: if (assigned(fOnUserSignal)) and (fUserSignalFilename<>'') then
                 spath := ForceConcatPath(Uploadpath,fUserSignalFilename)
              else
                 continue;
       end;
       if FileExists(spath) then { has a signal }
       begin
         if FileExists(StatusPath) then { so we can use its eventual creation as a "done" signal }
            DeleteFile(StatusPath);
         sleep(2000); { give time for the signal file to be closed }
         if not FileExists(spath) then
            continue; { just in case another instance dealt with it! }
         if fcount = 1 then { Gather }
           checkForGatherParams(spath); { are they trying to tell us something via the file contents? }
         DeleteFile(spath); { acknowledge start of operation }
         case fcount of
              0: Deploy;
              1: Gather;
              2: Update;
              3: fOnUserSignal(Self);
         end;
       end;
     end; { fcount }
   end; { sites }
  finally
   Site := saveSite; { keep default to keep UI in sync basically }
   fScanningSites := False;
  end; { try }
end;

// the main logic:

function TWebCompress.GetStatus: TStrings;
begin
 result := fStatus;
 if FileExists(StatusPath) then
 begin
   fStatus.clear;
   try
     fStatus.LoadFromFile(StatusPath);
   except on Exception do exit;
   end;
 end;
end;


procedure TWebCompress.Gather;
var results: TStringList;
    count: Integer;
begin
 if (not Init) or
  (GatherList.count = 0) or (Site='') or (not isDir(UploadPath)) then
    exit;
 results := TStringlist.create;
 try
    with CompLHA do
    begin
      fSaveCheckFile := OnCheckFile;
      OnCheckFile := Checkfile;
      ArchiveName := ForceConcatPath(UploadPath,'Gather.lzh');
      Confirm := False;
      ExceptionOnFileError := True;
      FilesToProcess.clear;
      Password := fPassword;
      StorePath := True;
      TargetPath := OriginPath; { force stored paths to be made relative to OriginPath }
      for count := 0 to pred(fGatherList.count) do
        FilesToProcess.add(RightofEqual(fGatherList[count]));
      { properties we DON'T tamper with, but which should be set
        after consideration by the user are:
        CompressionMethod (recommended colh5)
        ArchiveType (recommended caLHA)
        Regname/RegNumber (of course)
      }

      if assigned(OnGather) then
         OnGather(FilesToProcess); { allow them to tweak list if needed }
      if FilesToProcess.count=0 then exit; { in case they changed it }
      results.add('Gathering '+IntToStr(FilesToProcess.count)+' filespecs into '+
          ArchiveName+' at '+DateTimeToStr(now));
      try
        fcheckMessages.clear;
        Compress;
      except on E:Exception do
        begin
          results.add(E.message);
          results.add(ArchiveName+' -- compression failed -- processing aborted');
          exit;
        end;
      end;
      results.addStrings(fcheckMessages);
      results.addStrings(filesprocessed);
      results.add(CompLHA.archiveName+' compression completed ('+IntToStr(filesprocessed.count)+' files)');
    end; { with }

    SetStatusStrings(results);
    if assigned(AfterGather) then
        AfterGather(CompLHA.FilesProcessed);
    fStatus.SaveToFile(StatusPath);
 finally
    CompLHA.OnCheckFile := FSaveCheckFile;
    CompLHA.Password := '';
    results.free;
 end;
end;

procedure TWebCompress.Deploy;
var results: TStringList;
    files: TStringList;
    filename: string;
    processed, count : Integer;
begin
 if (not Init) or (Site='') or (not isDir(UploadPath)) then
    exit;

 results := TStringList.create;

 try
   processed := 0;
   files := TStringList.create;
   files.sorted := true; { should take care of order as files are yy-mm-dd.nnn format }
   try
    CompLHA.GetMatchingFiles(files,ForceConcatPath(UploadPath,'*.0*'));
    if assigned(OnDeploy) then
       OnDeploy(files); { allow them to tweak if needed }
    results.add('Deploying '+IntToStr(files.count)+' archives from '+uploadpath+
          ' at '+DateTimeToStr(now));
    for count := 0 to pred(files.count) do { got any? -- processes in sorted order }
    begin
       filename := files[count];
       if assigned(OnExpand) then
          OnExpand(filename,results);
       if FileExists(filename) then { still a valid file... }
       begin
         if not ExpandOneArchive(filename,results) then
           break; { do no more after a failure }
         inc(processed);
         if assigned(AfterExpand) then
            AfterExpand(filename,CompLHA.FilesProcessed,results);
       end;
    end; { for }
   finally
    files.free;
   end;
   SetStatusStrings(results);
   if assigned(AfterDeploy) then
      AfterDeploy(processed,files);
   fStatus.SaveToFile(StatusPath);
 finally
    results.free;
 end;

end; { Deploy }

function TWebCompress.ExpandOneArchive(const filename: string;results: TStrings): Boolean;
begin
   result := False;
   with CompLHA do
   try
      fSaveCheckFile := OnCheckFile;
      OnCheckFile := Checkfile;
      ArchiveName := filename;
      Confirm := False;
      FilesToProcess.clear;
      Password := fPassword;
      ExceptionOnFileError := True;
      if OriginPath <> '' then // v1.5, test replaces direct assignment
          TargetPath := OriginPath
      else
        TargetPath := extractFilePath(fileName); // new spec 1.5 if OriginPath is blank

      { properties we DON'T tamper with, but which should be set
        after consideration by the user are:
        MakeDirectories (recommended True)
        Regname/RegNumber (of course)
      }
      try
        Verify;
        FilesToProcess.clear; { in case set by previous operation }
        fCheckMessages.Clear;
        Expand;
      except on E:Exception do
        begin
          results.add(E.message);
          results.add(Archivename+' -- expansion failed -- processing aborted');
          exit;
        end;
      end;
      results.addStrings(fcheckMessages);
      results.addStrings(filesprocessed);
      results.add(filename+' expansion completed');
      DeleteFile(filename); { clean up when done... }
      result := True;
   finally
     OnCheckFile := FSaveCheckFile;
     Password :='';
   end;
end; { ExpandOneArchive }

procedure Register;
begin
  RegisterComponents('Compress', [TWebCompress]);
end;

end.
