unit whcompr;

(*  TWHCompress V2.0
   (c) 1997-8 South Pacific Information Services Ltd
    ALL RIGHTS RESERVED     software@spis.co.nz

Legal stuff:
This source is provided in support of our remote administration
applications for Web sites. It may not be used or modified for the
development of similar components or applications for Web site
developers or users without the express permission of SPIS Ltd.

TWHCompress is a compression automation and administration
WebAction component for WebHub(tm). It requires our TCompLHA component
(http://www.spis.co.nz/complha.htm) to provide the compression
services. It provides a powerful HTML-controllable interface for
specifying either:

* Uploaded archives to be deployed on the site (or sites) which
  your application is managing (e.g. new pages, chunk files,
  INI files, images, DB files etc)
* Files from the site which are to be "gathered" into an
  archive for subsequent downloading (e.g. stats, order
  information, messages, customer information, site backup)

Full verification of archives is implemented, to prevent failures
due to erroneous uploads. File encryption is also supported.

You'll probably also want to obtain our WebQueue application, which is
designed to interact with TWebCompress via FTP -- it will
automatically create an FTP update files for TWebCompress to
deploy, or specify files for TWebCompress to gather, then fetch
the resulting archive. WebQueue is NOT essential, because
TWebCompress will process any archives made by the popular
LHA freeware utility. HOWEVER, once you've tried WebQueue, we
doubt you've ever go back to LHA!

Thus, the administration of all remote site maintenance can be
reliably automated with a combination of TWebCompress+TCompLHA
and optionally, WebQueue.

This is V2.0, which is adequate for updating a one-EXE site. It
can be used on multi-EXE sites provided you don't need to
overwrite files which the other EXEs might have open.

Multi-EXE enhancements in future versions depend on features
which are promised for WebHub, viz: the ability to
request WebApps to temporarily suspend processing (including
closing files/tables etc) to allow the updates to take place.

See wcPanel.pas for a working example configured as a "drop in"
Webhub panel. WebCompr.txt contains full component documentation,
and WebQueue.hlp has information on the WebQueue utility.

*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UpdateOk, tpAction, WebTypes, WebIniFL, WebLink, CompLHA, Webcall, WebIList,
  WebList;

type
  TOnDeployEvent = procedure(files: TStrings) of Object;
  TAfterDeployEvent = procedure(processed: Integer;files: TStrings) of Object;
  TOnExpandEvent = procedure(var filename: string; status: TStrings) of Object;
  TAfterExpandEvent = procedure(const filename: string; filesprocessed, status: TStrings) of Object;
  TAfterChangePropertyEvent = procedure(const propertyname: string) of Object;

  TWHCompress = class(TWebAction)
  private
   fcommandline: TWebCommandLine;
   fSites: TWebINIList;
   fSite: string;
   fUserSignalFileName: string;
   fUploadPath: string;
   fAppID: string;
   fPassword: string;
   fOriginPath: string;
   fCheckMessages: TStringList;
   fStatus: TStrings;
   fGatherList: TWebIniList;
   fCompLHA: TCompLHA;
   fResponsive: Boolean;
   fAllowWebAdmin: Boolean;
   fRenameEXEs: Boolean;
   fAfterChangeProperty: TAfterChangePropertyEvent;
   fOnDeploy: TOnDeployEvent;
   fAfterDeploy: TAfterDeployEvent;
   fOnGather: TOnDeployEvent;
   fAfterGather: TOnDeployEvent;
   fOnExpand: TOnExpandEvent;
   fAfterExpand: TAfterExpandEvent;
   fSaveCheckFile: TCompLHAFilenameCheckEvent;
   fScanningSites: Boolean;
   fOnUserSignal : TNotifyEvent;
   procedure SetUserSignal(const Value: String);
   function GetStatus: TStrings;
   procedure Deploy;
   procedure Gather;
   procedure CheckFile(const originalFilePath: string;var newFilepath: String; dateTime: Longint;
           mode: TCompLHAProcessMode);
   function ExpandOneArchive(const filename: string;results: TStrings): Boolean;
   function GetTextArea(ListProperty: TStrings;const propertyName: string): TStrings;
   procedure AssignFromTextArea(ListProperty: TStrings;const propertyName: string);
   procedure checkForGatherParams(const spath: string); { are they trying to tell us something via the file contents? }
   function StatusPath: string;

  protected
    procedure DoExecute; override;
    function DoUpdate: Boolean; override;

    function  GetLocalVerbCount: Integer;
    function  GetVerbCount: Integer; Override;
    function  GetVerb(Index: Integer): string; Override;
    procedure ExecuteVerb(Index: Integer); Override;

    procedure ReadFromIniFile(const NewName,aName:String);
    procedure WriteToIniFile;
    procedure ReadSiteFromINIFile;
    procedure SetName(const NewName: TComponentName); override;
    function init: Boolean;
    procedure SetStatusStrings(Value: TStringList);
    procedure SetGatherStrings(Value: TWebINIList);
    procedure SetNoStrings(value: TStrings);
    procedure SetSites(Value: TWebINIList);
    procedure SetSite(Value: string);
    procedure SetOriginPath(const Value: String);
    procedure SetAppID(const Value: String);
    procedure SetPassWord(const Value: String);
    procedure SetUpLoadPath(const Value: string);
    procedure DoPropertyChange(const propertyname: string);

  public
   constructor Create(AOwner: TComponent); override;
   destructor  Destroy; override;
   procedure ApplicationIdle(Sender: TObject; var Done: Boolean);
   function SendListPropertyNoBr(const componentproperty: string): Boolean;

  published
   property WebCommandLine: TWebCommandLine read fCommandLine write fCommandLine;
   property UploadPath: string read fUploadPath write SetUploadPath stored False;
   property OriginPath: string read fOriginPath write SetOriginPath stored False;
   property AppID: string read fAppID write SetAppID stored False;
   property Password: string read fPassword write SetPassword stored False;
   property Site: string read fSite write SetSite stored false;
   property Sites: TWebINIList read fSites write SetSites stored false;
   property CompLHA: TCompLHA read fCompLHA write fCompLHA;
   property GatherList: TWebIniList read fgatherList write SetGatherStrings stored false;
   property Responsive: Boolean read fResponsive write fResponsive;
   property AllowWebAdmin: Boolean read fAllowWebAdmin write fAllowWebAdmin;
   property RenameEXEs: Boolean read fRenameEXEs write fRenameEXEs default True;
   property OnDeploy:  TOnDeployEvent read fOnDeploy write fOnDeploy;
   property AfterDeploy: TAfterDeployEvent read fAfterDeploy write fAfterDeploy;
   property AfterChangeProperty:  TAfterChangePropertyEvent read fAfterChangeProperty write fAfterChangeProperty;
   property OnExpand: TOnExpandEvent read fOnExpand write fOnExpand;
   property AfterExpand: TAfterExpandEvent read fAfterExpand write fAfterExpand;
   property OnGather:  TOnDeployEvent read fOnGather write fOnGather;
   property AfterGather:  TOnDeployEvent read fAfterGather write fAfterGather;
   property Status: TStrings read GetStatus write SetNoStrings;
   property UserSignalFileName: string read fUserSignalFileName write SetUserSignal;
   property OnUserSignal: TNotifyEvent read fOnUserSignal write fOnUserSignal;
  end;

procedure Register;

implementation

uses ucstring, { various }
     ucode, { Pause }
     ucFile, { ConCatPath }
     typinfo; { SendListPropertyNoBr }

const { INI settings }
      SITENAME='Site'; { under [WebCompress1] }
      SITESNAME='Sites'; { e.g. [WebCompress1.Sites] }
      USERSIGNALNAME='UserSignal';

      PASSWORDNAME='Password'; { under [WebCompress1.Sitename] }
      APPIDNAME='AppID';
      ORIGINPATHNAME='OriginPath';
      UPLOADPATHNAME='UploadPath';

      GATHERLISTNAME='GatherList'; { e.g. WebCompress1.Sitename.GatherList }

      SELECTSITECOMMAND = 'SelectSite';
      EXECUTECOMMAND='Execute';
      GATHERCOMMAND='Gather';
      DEPLOYCOMMAND='Deploy';
      REFRESHCOMMAND='Refresh';

{ We'll *always* prepend a non-blank path provided Filename doesn't have a
  leading UNC or drive designator. This differs from Michael's code which
  skips the prepend if the filename already has an absolute path.
}
function ForceConCatPath(const Path,FileName:String):String;
begin
 result := Filename;
 if Length(result)>0 then
 begin
   if Path<>'' then { may need to adjust filename }
   begin
     if result[1]='\' then    { is a leading \ }
       if copy(result,1,2)<>'\\' then { but not a UNC path }
         result :=copy(result,2,999); { strip the leading \ }
   end;
 end;
 result := ConCatPath(Path,Result); { tidily prepend... }
end;

function isDir(const path: string): Boolean;
var attr: Integer;
begin
  attr:=FileGetAttr(path);
  result := (attr>=0) and ((attr AND faDirectory) > 0);
end;

function TWHCompress.SendListPropertyNoBr(
           const ComponentProperty: string): Boolean;
var PropInfo: PPropInfo;
    i: Integer;
    componentName: string;
    propertyName: string;
    ActionComponent, o: TObject;
begin
  result := False;
  splitstring(ComponentProperty,'.', { e.g. WebCompress.GatherList }
      componentName,PropertyName);
  i:=webinfo.webOpenActionList.findbykey(ComponentName);
  if i < 0 then
   exit; { not a Webaction component, forget it }

  ActionComponent:=twebaction(Webinfo.webOpenActionList.objects[i]);

  PropInfo:=GetPropInfo(ActionComponent.ClassInfo,PropertyName);
  if PropInfo=nil then
    exit;
  if (PropInfo^.PropType^.Kind=tkClass) then
  begin
    o:=TObject(GetOrdProp(ActionComponent, PropInfo));
    if (o<>nil) and (o is TStrings) then
    begin
      Result := True;
      WebOutput.SendStringListNoBr(o as TStrings);
    end;
  end;
end;

procedure TWHCompress.CheckFile(const originalFilePath: string;var newFilepath: String; dateTime: Longint;
           mode: TCompLHAProcessMode);
var ext, diskpath: string;
    relative: Boolean;
begin
  { start by ensuring all access to and from is relative to (or at least
    matching) OriginPath
  }
  if OriginPath<>'' then { ok, do some work -- expand OR compress }
  begin
     relative := NewFilePath[1]<>'\';
     if mode = cmLHAExpand then
     begin
       if (pos(':',OriginPath)>0) and (not relative) then { get drive }
         NewFilepath := ForceConcatPath(copy(OriginPath,1,pos(':',OriginPath)),NewFilePath);
       diskPath := newFilePath;
     end else
       diskPath := OriginalFilePath; { where we're coming FROM }
     ext := copy(diskpath,1,Length(OriginPath));
     if CompareText(ext,OriginPath)<>0 then { not in the same path, too bad... }
     begin { illegal }
         if not CompLHA.safemode then { ignore if Verify step... }
            fCheckMessages.add('=== '+diskPath+' NOT expanded -- illegal path!');
         newFilePath := CompLHASkipFlag; { no way, Jose }
         exit;
     end;
  end;

  if (mode=cmLHAExpand) and RenameEXEs then
  begin
    ext := UpperCase(ExtractFileExt(originalFilePath));
    if (ext='.EXE') or (ext='.OCX') or (ext='.DLL') then
    begin
       ext := copy(ext,1,3)+'_'; { EXE --> EX_ }
       newfilepath := ChangeFileExt(newfilepath,ext);
       if not CompLHA.safemode then { ignore if Verify step... }
          fCheckMessages.add('=== '+originalFilePath+' RENAMED to '+newfilePath);
    end;
  end;

  if assigned(FSaveCheckFile) then
     FSaveCheckFile(originalFilePath,newFilePath,dateTime,mode);
end;

constructor TWHCompress.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  fStatus := TStringList.create;
  fGatherList := TWebINIList.createFor(Self,GatherListName);
  fSites := TWebINIList.createFor(Self,SitesName);
  fCheckMessages := TStringList.Create;
  RenameExes := True;
end;

destructor TWHCompress.Destroy;
begin
  fStatus.free;
  fGatherList.free;
  fSites.free;
  fCheckMessages.free;
  inherited Destroy;
end;

function TWHCompress.init: Boolean;
begin
  result := assigned(Webapp) and assigned(WebCommandLine) and
            assigned(CompLHA);
end;

procedure TWHCompress.DoPropertyChange(const propertyname: string);
begin
  if fScanningSites then exit; { ignore when we're looking through sites in the idle loop }
  if assigned(fAfterChangeProperty) then
    fAfterChangeProperty(propertyname);
end;

procedure TWHCompress.SetSites(Value: TWebINIList);
begin
  fSites.assign(value);
  WriteToINIFile;
  DoPropertyChange('Sites');
end;

procedure TWHCompress.SetSite(Value: string);
var count: Integer;
    newsite: string;
begin
  newsite := '';
  for count := 0 to pred(Sites.count) do
  begin
    if CompareText(RightOfEqual(Sites[count]),value)=0 then
    begin
       newsite := RightofEqual(Sites[count]);
       break;
    end;
  end;
  if fSite<>newsite then
  begin
   fSite := newsite;
   DoPropertyChange('Site');
   ReadSiteFromINIFile; { get all properties up to date }
  end;
end;

procedure TWHCompress.SetUserSignal(const Value: String);
begin
  fUserSignalFileName := value;
  WriteToIniFile;
  DoPropertyChange('UserSignal');
end;

procedure TWHCompress.SetAppID(const Value: String);
begin
  fAppID := value;
  WriteToIniFile;
  DoPropertyChange('AppID');
end;

procedure TWHCompress.SetPassword(const Value: String);
begin
  fPassword := value;
  WriteToIniFile;
  DoPropertyChange('Password');
end;


procedure TWHCompress.SetUploadPath(const Value: String);
begin
  fUploadPath := value;
  WriteToIniFile;
  DoPropertyChange('UploadPath');
end;


procedure TWHCompress.SetOriginPath(const Value: String);
begin
  fOriginPath := value;
  WriteToIniFile;
  DoPropertyChange('OriginPath');
end;

function TWHCompress.StatusPath: string;
begin
  result := ForceConcatPath(UploadPath,'STATUS.TXT');
end;

procedure TWHCompress.SetStatusStrings(Value: TStringList);
begin
 fStatus.Assign(Value);
 if FileExists(StatusPath) then
   DeleteFile(StatusPath);
end;

procedure TWHCompress.SetGatherStrings(Value: TWebINIList);
begin
  fGatherList.Assign(Value);
  WriteToINIFile;
  DoPropertyChange('GatherList');
end;

function TWHCompress.GetTextArea(ListProperty: Tstrings;const propertyName: string): TStrings;
var id: string;
begin
   result := nil;
   if WebSession=nil then exit;
   id :='txt'+Name+PropertyName;    { e.g TxTWHCompress1GatherList  }
   if WebSession.TextAreas.FindByKey(ID)<0 then { no such list }
      exit; { nil result means no existing text area of that name }
   result := WebSession.TextAreas.List[id];
end;

procedure TWHCompress.AssignFromTextArea(ListProperty: TStrings;const propertyName: string);
var TextAreaStrings: TStrings;
    lcount: Integer;
begin
   textAreaStrings := GetTextArea(ListProperty,propertyName);
   if TextAreaStrings <> nil then { we ONLY want to update it if the TextArea exists!!! }
   begin
      if ListProperty is TWebINIList then
      begin
        ListProperty.Clear;
        for lcount := 0 to pred(TextAreaStrings.count) do
          ListProperty.add(IntToStr(succ(lcount))+'='+
              RightOfEqual(TextAreaStrings[lcount]));
        WriteToINIFile;
      end else { regular string list property }
        ListProperty.assign(TextAreaStrings);
   end;
end;

function TWHCompress.DoUpdate:Boolean;
begin
  result := inherited DoUpdate and
           not((WebApp=nil) or (WebCommandLine = nil) or (CompLHA=nil));
  if Result then
    ReadFromIniFile(Name,'');
end;

procedure TWHCompress.SetName(const NewName: TComponentName);
begin
  if assigned(WebApp) and (NewName<>Name) then
    ReadFromIniFile(NewName,Name); {now renaming in designer}
  inherited SetName(NewName);
end;

function TWHCompress.GetLocalVerbCount: Integer;
begin
  Result := 3;
end;

function TWHCompress.GetVerbCount: Integer;
begin
  Result := inherited GetVerbCount + GetLocalVerbCount;
end;

function TWHCompress.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'Edit GatherList';
    1: Result := 'Edit Sites';
    2: Result := 'Gather';
  else
    Result:=inherited GetVerb(Index - GetLocalVerbCount);
  end;
end;

procedure TWHCompress.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: begin
         GatherList.Edit;
         DoPropertyChange('GatherList');
       end;
    1: begin
         Sites.Edit;
         DoPropertyChange('Sites');
       end;
    2: Gather;
  else
    Inherited ExecuteVerb(Index - GetLocalVerbCount);
    end;
end;

procedure TWHCompress.WriteToIniFile;
begin
  if (not assigned(WebApp)) or (Site='') then
     exit;
  if (WebApp.WebIni=nil) then
    raise TWebException.Create(WebApp,'WebIni must not be blank');
  with WebApp.WebIni do begin
    IniFileName:=WebApp.IniFileName;
    Open;
    Section := Self.Name;
    stringEntry[SiteName] := Site;
    stringEntry[UserSignalName] := UserSignalFileName;
    Section := Self.name+'.'+Site;
    stringEntry[OriginPathName]:= OriginPath;
    stringEntry[UploadPathName]:= UploadPath;
    stringEntry[AppIDName]:= AppID;
    stringEntry[PasswordName]:= Password;
    Close;
    fGatherList.UpdateINIFile;
    fSites.UpdateINIFile;
  end;
end;

procedure TWHCompress.ReadFromIniFile(const NewName,aName:String);
var count: Integer;
begin
  if not assigned(WebApp) then
    exit;
  if (WebApp.WebIni=nil) then
    raise TWebException.Create(WebApp,'WebIni must not be blank');
  IniFileName := Webapp.IniFilename; { so right-click Edit INI works }
  with WebApp.WebIni do begin
    IniFileName:=WebApp.IniFileName;
    Open;
    LeaveOpen:=True;
    if (NewName<>aName) and (aName<>'') then begin
      Handle.EraseSection(aName); { WebCompress1 }
      Open;
      Handle.EraseSection(aName+'.'+SitesName); { WebCompress1.Sites }
      Open;
      for count := 0 to pred(fSites.count) do { remove old GatherLists }
      begin
        Handle.EraseSection(aName+'.'+RightofEqual(fSites[count])+'.'+GatherListName);
        Open;
      end;
      fSites.section := newname+'.'+SitesName;
      fSites.clear; { leaves closed... }
      end
    else begin
      fSites.ReadSection(WebApp,NewName+'.'+SitesName);
      Section := NewName; { e.g. WebCompress1 }
      UserSignalFileName:= stringEntry[UserSignalName];
      Site := StringEntry[SITENAME]; { default/current site, never empty if any defined... }
           (* which in and of itself forces read from INI file... *)
      Close;
     end; { else }
    end;
end;

procedure TWHCompress.ReadSiteFromINIFile;
begin
  if (not assigned(WebApp)) or (Site='') then
     exit;
  if (WebApp.WebIni=nil) then
    raise TWebException.Create(WebApp,'WebIni must not be blank');
  with WebApp.WebINI do
  begin
    fGatherList.section :=Self.Name+'.'+Site+'.'+GatherListName; { ensure set for writing too }
    fGatherList.ReadSection(WebApp,fGatherList.section);
    Section := Self.Name+'.'+Site;
    fOriginPath := stringEntry[OriginPathName];
    fUploadPath := stringEntry[UploadPathName];
    fAppID := stringEntry[AppIDName];
    fPassword := stringEntry[PasswordName];
    DoPropertyChange('GatherList');
    DoPropertyChange('OriginPath');
    DoPropertyChange('UploadPath');
    DoPropertyChange('AppID');
    DoPropertyChange('Password');
  end;
end;


procedure TWHCompress.checkForGatherParams(const spath: string);
var lines: TStringList;
    count: Integer;
    line: string;
    cleared: Boolean;
begin { check if signal file has CONTENT, being GatherList settings... }
  lines := TStringList.Create;
  cleared := False;
  try
    try
      lines.LoadFromFile(spath);
    except on Exception do exit;
    end;
    for count := 0 to pred(lines.count) do
    begin
       line := lines[count];
       if line='' then continue;
       if not Cleared then
          GatherList.clear;
       Cleared := True;
       GatherList.add(inttostr(succ(count))+'='+line); { put the (presumed) paths into the list }
    end;
    if Cleared then { changed GatherList }
    begin
      WriteToINIFile;
      DoPropertyChange('GatherList');
    end;
  finally
    lines.free;
  end;
end;

procedure TWHCompress.ApplicationIdle(Sender: TObject;
  var Done: Boolean);
var spath, savesite: string;
    sitecount, fcount: Integer;
begin

  if Responsive then
  begin
   Done := False;
   sleep(100);
  end else { easy on the CPU, but veerry slow to respond }
   Done := True;

  try
   WebCommandLine.active := False; { suspend page requests }
   fScanningSites := True; { suspend AfterPropertyChange calls for now... }
   savesite := fSite;
   for sitecount := 0 to pred(Sites.count) do
   begin
     Site := RightofEqual(Sites[sitecount]); { load settings from INI }
     if not isDir(UploadPath) then continue; { ignore bad paths }
     for fcount := 0 to 2 do
     begin
       case fcount of
           0: spath := ForceConcatPath(Uploadpath,DeployCommand+'.SIG');
           1: spath := ForceConcatPath(Uploadpath,GatherCommand+'.SIG');
           2: spath := ForceConcatPath(Uploadpath,RefreshCommand+'.SIG');
           3: if (assigned(fOnUserSignal)) and (fUserSignalFilename<>'') then
                 spath := ForceConcatPath(Uploadpath,fUserSignalFilename)
              else
                 continue;
       end;
       if FileExists(spath) then { has a signal }
       begin
         if FileExists(StatusPath) then { so we can use its eventual creation as a "done" signal }
            DeleteFile(StatusPath);
         sleep(2000); { give time for the signal file to be closed }
         if not FileExists(spath) then
            continue; { just in case another instance dealt with it! }
         if fcount = 1 then { Gather }
           checkForGatherParams(spath); { are they trying to tell us something via the file contents? }
         DeleteFile(spath); { acknowledge start of operation }
         case fcount of
              0: Deploy;
              1: Gather;
              2: DoUpdate;
              3: fOnUserSignal(Self);
         end;
       end;
     end; { fcount }
   end; { sites }
  finally
   WebCommandLine.active := True; { resume page requests }
   Site := saveSite; { keep default to keep UI in sync basically }
   fScanningSites := False;
  end; { try }
end;

// The guts...

procedure TWHCompress.DoExecute;
var cmd, Sessvar, sitename : string;
begin
   if tpUpdated and Init then begin
    inherited DoExecute;

   splitstring(Command,'|',cmd,sitename);
   if sitename<>'' then
     Site := WebApp.CompareTo(sitename); { get the site parameter, if any, allow macro expansion... }
   if Site='' then { not valid site! }
       raise TWebException.Create(WebApp,'"'+sitename+'" is not a valid site name!');

   if WebSession<> nil then
   begin
     with WebSession.Literals do
     begin
       if AllowWebAdmin then
       begin
         sessvar := webapp.literalState[Self,UploadPathName,UploadPath];
                   // -- keeps UploadPath unless matching literal is found...
         if (sessvar='') or (not isDir(sessvar)) then
             raise TWebException.Create(WebApp,sessvar+' is not a valid upload path!');
         UploadPath:=sessvar;

         sessvar := webapp.literalState[Self,OriginPathName,OriginPath];
         if (sessvar='') or (not isDir(sessvar)) then
            raise TWebException.Create(WebApp,sessvar+' is not a valid origin path!');
         OriginPath:=sessvar;

         AppID := webapp.literalState[Self,AppIDName,AppID];
         PassWord := webapp.literalState[Self,PasswordName,Password];
         if Password='-' then
            Password:=''; { can't clear from HTML otherwise... }

         AssignFromTextArea(Sites,'Sites'); { if set }
         AssignFromTextArea(GatherList,'GatherList');
       end; { allowWebAdmin }
     end; { with }
   end; { if has a WebSession }

   if CompareText(cmd,GatherCommand)=0 then
     Gather
   else if (CompareText(cmd,SelectSiteCommand)=0) then
     exit  { job is presumably already done -- it's just so we can edit the vars above }
   else if (cmd='') or (CompareText(cmd,ExecuteCommand)=0) or
       (CompareText(cmd,DeployCommand)=0) then
     Deploy;
   end;
end;
 // the main logic:

function TWHCompress.GetStatus: TStrings;
begin
 result := fStatus;
 if FileExists(StatusPath) then
 begin
   fStatus.clear;
   try
     fStatus.LoadFromFile(StatusPath);
   except on Exception do exit;
   end;
 end;
end;

procedure TWHCompress.SetNoStrings(value: TStrings);
begin
{do nothing mate -- just permits Status to be public, hence HTML callable }
end;



procedure TWHCompress.Gather;
var results: TStringList;
    count: Integer;
begin
 if (not Init) or (not tpUpdated) or
  (GatherList.count = 0) or (Site='') or (not isDir(UploadPath)) then
    exit;
 results := TStringlist.create;
 WebCommandLine.active := False; { suspend page requests }
 try
    with CompLHA do
    begin
      fSaveCheckFile := OnCheckFile;
      OnCheckFile := Checkfile;
      ArchiveName := ForceConcatPath(UploadPath,'Gather.lzh');
      Confirm := False;
      ExceptionOnFileError := True;
      FilesToProcess.clear;
      Password := fPassword;
      StorePath := True;
      TargetPath := OriginPath; { force stored paths to be made relative to OriginPath }
      for count := 0 to pred(GatherList.count) do
        FilesToProcess.add(RightofEqual(GatherList[count]));
      { properties we DON'T tamper with, but which should be set
        after consideration by the user are:
        CompressionMethod (recommended colh5)
        ArchiveType (recommended caLHA)
        Regname/RegNumber (of course)
      }

      if assigned(OnGather) then
         OnGather(FilesToProcess); { allow them to tweak list if needed }
      if FilesToProcess.count=0 then exit; { in case they changed it }
      results.add('Gathering '+IntToStr(FilesToProcess.count)+' filespecs into '+
          ArchiveName+' at '+DateTimeToStr(now));
      try
        fcheckMessages.clear;
        Compress;
      except on E:Exception do
        begin
          results.add(E.message);
          results.add(ArchiveName+' -- compression failed -- processing aborted');
          exit;
        end;
      end;
      results.addStrings(fcheckMessages);
      results.addStrings(filesprocessed);
      results.add(CompLHA.archiveName+' compression completed ('+IntToStr(filesprocessed.count)+' files)');
    end; { with }

    SetStatusStrings(results);
    if assigned(AfterGather) then
        AfterGather(CompLHA.FilesProcessed);
    fStatus.SaveToFile(StatusPath);
 finally
    CompLHA.OnCheckFile := FSaveCheckFile;
    CompLHA.Password := '';
    WebCommandLine.active := True; { reactivate page requests }
    results.free;
 end;
end;

procedure TWHCompress.Deploy;
var results: TStringList;
    files: TStringList;
    filename: string;
    processed, count : Integer;
begin
 if (not Init) or (not tpUpdated) or (Site='') or (not isDir(UploadPath)) then
    exit;

 results := TStringList.create;
 WebCommandLine.active := False; { suspend page requests }

 try
   processed := 0;
   files := TStringList.create;
   files.sorted := true; { should take care of order as files are yy-mm-dd.nnn format }
   try
    CompLHA.GetMatchingFiles(files,ForceConcatPath(UploadPath,'*.0*'));
    if assigned(OnDeploy) then
       OnDeploy(files); { allow them to tweak if needed }
    results.add('Deploying '+IntToStr(files.count)+' archives from '+uploadpath+
          ' at '+DateTimeToStr(now));
    for count := 0 to pred(files.count) do { got any? -- processes in sorted order }
    begin
       filename := files[count];
       if assigned(OnExpand) then
          OnExpand(filename,results);
       if FileExists(filename) then { still a valid file... }
       begin
         if not ExpandOneArchive(filename,results) then
           break; { do no more after a failure }
         inc(processed);
         if assigned(AfterExpand) then
            AfterExpand(filename,CompLHA.FilesProcessed,results);
       end;
    end; { for }
   finally
    files.free;
   end;
   SetStatusStrings(results);
   if assigned(AfterDeploy) then
      AfterDeploy(processed,files);
   fStatus.SaveToFile(StatusPath);
 finally
    WebCommandLine.active := True; { reactivate page requests }
    results.free;
 end;

end; { Deploy }

function TWHCompress.ExpandOneArchive(const filename: string;results: TStrings): Boolean;
begin
   result := False;
   with CompLHA do
   try
      fSaveCheckFile := OnCheckFile;
      OnCheckFile := Checkfile;
      ArchiveName := filename;
      Confirm := False;
      FilesToProcess.clear;
      Password := fPassword;
      ExceptionOnFileError := True;
      if OriginPath <> '' then // v1.5, test replaces direct assignment
          TargetPath := OriginPath
      else
        TargetPath := extractFilePath(fileName); // new spec 1.5 if OriginPath is blank

      { properties we DON'T tamper with, but which should be set
        after consideration by the user are:
        MakeDirectories (recommended True)
        Regname/RegNumber (of course)
      }
      try
        Verify;
        FilesToProcess.clear; { in case set by previous operation }
        fCheckMessages.Clear;
        Expand;
      except on E:Exception do
        begin
          results.add(E.message);
          results.add(Archivename+' -- expansion failed -- processing aborted');
          exit;
        end;
      end;
      results.addStrings(fcheckMessages);
      results.addStrings(filesprocessed);
      results.add(filename+' expansion completed');
      DeleteFile(filename); { clean up when done... }
      result := True;
   finally
     OnCheckFile := FSaveCheckFile;
     Password :='';
   end;
end; { ExpandOneArchive }

procedure Register;
begin
  RegisterComponents('Compress', [TWHCompress]);
end;

end.
